Ext.namespace('Zarafa.plugins.threema4deskapp');

Zarafa.plugins.threema4deskapp.isDeskApp = function(){
	return Ext.isFunction(Zarafa.isDeskApp) ? Zarafa.isDeskApp() : Ext.isDefined(window.nw);
};

/**
 * @class Zarafa.plugins.threema4deskapp.Threema4DeskApp
 * @extends Zarafa.core.Plugin
 *
 * Plugin that adds Threema to DeskApp
 */
Zarafa.plugins.threema4deskapp.Threema4DeskApp = Ext.extend(Zarafa.core.Plugin, {

	/**
	 * Initializes the plugin.
	 */
	initPlugin : function(){
		if ( Zarafa.plugins.threema4deskapp.isDeskApp() ){
			var pluginSettings = container.getSettingsModel().get('zarafa/v1/plugins/threema4deskapp', true);
			var site = {
				buttonText: pluginSettings['button-title'],
				url: pluginSettings.url,
				tabOrder: 10
			};

			// The tab in the top tabbar
			this.registerInsertionPoint('main.maintabbar.left', this.createMainTab.createDelegate(this, [site]), this);

			// The settings category
			this.registerInsertionPoint('context.settings.categories', this.createSettingCategory, this);

			// Register mail specific dialog types
			Zarafa.core.data.SharedComponentType.addProperty('plugins.threema4deskapp.panel');

			// Autostart if needed
			if ( pluginSettings.autostart ){
				container.on('webapploaded', function(){
					// use the openTab function for convenience
					this.openTab(this.createMainTab(site));

					// OpenTab will also switch to the Threema tab, but we don't want that, so
					// let's switch back to the main tab.
					var mainContentTabPanel = container.getMainPanel().contentPanel;
					mainContentTabPanel.activate(0);
				}, this);
			}
		}
	},

	/**
	 * Adds a button to the top tab bar for this context.
	 * @return {Object} The button for the top tabbar
	 * @private
	 */
	createMainTab: function(site)
	{
		return {
			text: site.buttonText,
			url: site.url,
			tabOrderIndex: site.tabOrder,
			cls: 'mainmenu-button-threema4deskapp',
			handler: this.openTab
		};
	},

	/**
	 * Event handler for the click event of the tabbar buttons. It will
	 * open the tab if it already exists, or create it otherwise.
	 * @param {Zarafa.core.ui.MainTab} btn The button in the
	 * {@link Zarafa.core.ui.MainTabBar main tabbar}
	 */
	openTab: function(btn)
	{
		console.log('opening tab');
		var tabIndex;
		Ext.each(container.getTabPanel().items.items, function(item, index){
			if ( item.url===btn.url && item.title===btn.text ){
				tabIndex = index;
			}
		});

		if ( Ext.isDefined(tabIndex) ){
			// open the existing tab
			var mainContentTabPanel = container.getMainPanel().contentPanel;
			mainContentTabPanel.activate(tabIndex);

		} else {
			// Create a new tab
			Zarafa.core.data.UIFactory.openLayerComponent(
				Zarafa.core.data.SharedComponentType['plugins.threema4deskapp.panel'],
				null,
				{
					url: btn.url,
					title: btn.text,
					tabOrder: btn.tabOrderIndex
				}
			);
		}
	},

	createSettingCategory : function()
	{
		return {
			xtype: 'zarafa.plugins.threema4deskapp.settings.category'
		};
	},

	/**
	 * Bid for the type of shared component
	 * and the given record.
	 * This will bid on a common.dialog.create or common.dialog.view for a
	 * record with a message class set to IPM or IPM.Note.
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context
	 * can bid for.
	 * @param {Ext.data.Record} record Optionally passed record.
	 * @return {Number} The bid for the shared component
	 */
	bidSharedComponent : function(type, record)
	{
			var bid = -1;

			switch (type) {
				case Zarafa.core.data.SharedComponentType['plugins.threema4deskapp.panel']:
					bid = 1;
			}

			return bid;
	},

	/**
	 * Will return the reference to the shared component.
	 * Based on the type of component requested a component is returned.
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context
	 * can bid for.
	 * @param {Ext.data.Record} record Optionally passed record.
	 * @return {Ext.Component} Component
	 */
	getSharedComponent : function(type, record)
	{
		var component;
		switch (type)
		{
			case Zarafa.core.data.SharedComponentType['plugins.threema4deskapp.panel']:
				return Zarafa.plugins.threema4deskapp.ui.ContentPanel;
		}
	}

});

Zarafa.onReady(function() {
	container.registerPlugin(new Zarafa.core.PluginMetaData({
		name : 'threema4deskapp',
		displayName : _('Threema for DeskApp'),
		allowUserVisible : Zarafa.plugins.threema4deskapp.isDeskApp(),
		pluginConstructor : Zarafa.plugins.threema4deskapp.Threema4DeskApp
	}));
});
Ext.namespace('Zarafa.plugins.threema4deskapp.settings');

/**
 * @class Zarafa.plugins.threema4deskapp.settings.Category
 * @extends Zarafa.settings.ui.SettingsCategory
 * @xtype zarafa.plugins.threema4deskapp.settings.category
 */
Zarafa.plugins.threema4deskapp.settings.Category = Ext.extend(Zarafa.settings.ui.SettingsCategory, {
	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			title : _('Threema'),
			categoryIndex : 12,
			iconCls : 'k-threema4deskapp-settings-category',
			items : [{
				xtype : 'zarafa.plugins.threema4deskapp.settings.generalsettingswidget'
			}]
		});

		Zarafa.plugins.threema4deskapp.settings.Category.superclass.constructor.call(this, config);
	}
});

Ext.reg('zarafa.plugins.threema4deskapp.settings.category', Zarafa.plugins.threema4deskapp.settings.Category);
Ext.namespace('Zarafa.plugins.threema4deskapp.settings');

/**
 * @class Zarafa.plugins.threema4deskapp.settings.GeneralSettingsWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype zarafa.plugins.threema4deskapp.settings.generalsettingswidget
 */
Zarafa.plugins.threema4deskapp.settings.GeneralSettingsWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {
	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			cls: 'threema4deskapp-settings-panel zarafa-settings-widget',
			title: _('General settings'),
			items: [{
				xtype: 'checkbox',
				ref: 'autostart',
				hideLabel: true,
				boxLabel: _('Open Threema when DeskApp starts'),
				listeners: {
					check: this.onCheckAutoStart,
					scope: this
				}
			}]
		});

		Zarafa.plugins.threema4deskapp.settings.GeneralSettingsWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Event handler for the check event of the autostart checkbox.
	 *
	 * @param {Ext.form.Checkbox} checkbox The autostart checkbox
	 * @param {Boolean} checked True if the checkbox is checked, false otherwise
	 */
	onCheckAutoStart : function(checkbox, checked)
	{
		if ( !this.updating ){
			this.updateSettings(this.settingsModel);
		}
	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategoryWidgetPanel widget panel}
	 * to load the latest version of the settings from the
	 * {@link Zarafa.settings.SettingsModel} into the UI of this category.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to load
	 */
	update : function(settingsModel)
	{
		this.updating = true;
		this.settingsModel = settingsModel;

		var autostart = settingsModel.get('zarafa/v1/plugins/threema4deskapp/autostart')===true;
		this.autostart.setValue(autostart);

		this.updating = false;
	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategoryWidgetPanel widget panel}
	 * to update the settings from the UI into the {@link Zarafa.settings.SettingsModel settings model}.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to update
	 */
	updateSettings : function(settingsModel)
	{
		settingsModel.set('zarafa/v1/plugins/threema4deskapp/autostart', this.autostart.getValue());
	}
});

Ext.reg('zarafa.plugins.threema4deskapp.settings.generalsettingswidget', Zarafa.plugins.threema4deskapp.settings.GeneralSettingsWidget);
Ext.namespace('Zarafa.plugins.threema4deskapp.ui');

/**
 * @class Zarafa.plugins.threema4deskapp.ui.ContentPanel
 * @extends Zarafa.core.ui.ContentPanel
 */
Zarafa.plugins.threema4deskapp.ui.ContentPanel = Ext.extend(Zarafa.core.ui.ContentPanel, {
	/**
	 * @constructor
	 * @param config Configuration structure
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			// Overridden from Ext.Component
			xtype: 'zarafa.plugins.threema4deskapp.ui.contentpanel',
			layout : 'fit',
			header: false,
			iconCls: 'icon_threema',
			border: false,
			items : [{
				xtype: 'zarafa.plugins.threema4deskapp.ui.panel',
				url: config.url,
				tabOrder: config.tabOrder
			}]
		});

		Zarafa.plugins.threema4deskapp.ui.ContentPanel.superclass.constructor.call(this, config);
	}
});

Ext.reg('zarafa.plugins.threema4deskapp.ui.contentpanel', Zarafa.plugins.threema4deskapp.ui.ContentPanel);
Ext.namespace('Zarafa.plugins.threema4deskapp.ui');

/**
 * @class Zarafa.plugins.threema4deskapp.ui.Panel
 * @extends Ext.Panel
 */
Zarafa.plugins.threema4deskapp.ui.Panel = Ext.extend(Ext.Panel, {
	/**
	 * The id of the iframe element inside this panel
	 * @property
	 * @type String
	 */
	iframeId : undefined,

	/**
	 * @constructor
	 * @param config Configuration structure
	 */
	constructor : function(config)
	{
		config = config || {};

		this.iframeId = 'threema4deskapp-iframe-'+config.tabOrder;

		if (Zarafa.plugins.threema4deskapp.isDeskApp()) {
			this.tag = 'webview';
		} else {
			this.tag = 'iframe';
		}

		Ext.applyIf(config, {
			// Overridden from Ext.Component
			xtype: 'zarafa.plugins.threema4deskapp.ui.panel',
			layout : 'fit',
			header: false,
			border: false,
			iconCls: 'icon_threema',
			html : {
				tag: 'iframe',
				id: this.iframeId,
				cls: 'threema4deskapp-iframe',
				src: config.url
			},
			listeners: {
				scope: this,
				afterrender: this.onAfterRender
			}
		});

		Zarafa.plugins.threema4deskapp.ui.Panel.superclass.constructor.call(this, config);
	},

	/**
	 * Event handler for the afterrender event of the panel. Removes references to the main panel.
	 * Prevents the blur event listener that Threema sets to capture the focus. And injects some
	 * css styles into the iframe.
	 */
	onAfterRender : function()
	{
		var iframe = document.getElementById(this.iframeId);
		var self = this;
		iframe.addEventListener('load', function(){
			// For security remove references to the parent and nw, so threema cannot use those
			iframe.contentWindow.parent = null;
			iframe.contentWindow.opener = null;
			iframe.contentWindow.nw = null;

			// #HACK ALERT
			// Adding a nasty hack to avoid the focus capturing of Threema.
			// The first 'blur' event handler that is set, will do this, so we
			// will not accept that one.
			var origAddEventListener = iframe.contentWindow.EventTarget.prototype.addEventListener;
			var count = 0;
			iframe.contentWindow.EventTarget.prototype.addEventListener = function(event, handler){
				if ( event === 'blur' ){
					count++;
					if ( count === 1 ){
						return;
					}
				}
				return origAddEventListener.apply(this, arguments);
			};

			// Inject some styling
			var sheet = self.createStylesheet(iframe);
			sheet.insertRule(".app-wrapper-web .app { width: auto; height: 100%; top: 0; }", 0);
			sheet.insertRule(".pane-header { background-color: white; }", 0);
		});
	},

	/**
	 * Creates a stylesheet in the document of the iframe
	 * @param {HTMLElement} iframe The iframe element
	 *
	 * @return {Stylesheet} The created stylesheet
	 */
	createStylesheet : function(iframe)
	{
		// Create the <style> tag
		var style = iframe.contentDocument.createElement("style");

		// WebKit hack :(
		style.appendChild(iframe.contentDocument.createTextNode(""));

		// Add the <style> element to the page
		iframe.contentDocument.head.appendChild(style);

		return style.sheet;
	}
});

Ext.reg('zarafa.plugins.threema4deskapp.ui.panel', Zarafa.plugins.threema4deskapp.ui.Panel);
